searchData={"items":[{"type":"behaviour","title":"OffBroadway.Splunk.Client","doc":"A generic behaviour for implementing Splunk clients for\n`OffBroadway.Splunk.Producer`.\n\nThis module defines callbacks to normalize options and receive items\nfor a Splunk job.\n\nModules that implements this behaviour should be passed as the\n`:splunk_client` option from `OffBroadway.Splunk.Producer`.","ref":"OffBroadway.Splunk.Client.html"},{"type":"callback","title":"OffBroadway.Splunk.Client.ack_message/2","doc":"","ref":"OffBroadway.Splunk.Client.html#c:ack_message/2"},{"type":"callback","title":"OffBroadway.Splunk.Client.init/1","doc":"","ref":"OffBroadway.Splunk.Client.html#c:init/1"},{"type":"callback","title":"OffBroadway.Splunk.Client.receive_messages/3","doc":"","ref":"OffBroadway.Splunk.Client.html#c:receive_messages/3"},{"type":"callback","title":"OffBroadway.Splunk.Client.receive_status/2","doc":"","ref":"OffBroadway.Splunk.Client.html#c:receive_status/2"},{"type":"type","title":"OffBroadway.Splunk.Client.messages/0","doc":"","ref":"OffBroadway.Splunk.Client.html#t:messages/0"},{"type":"module","title":"OffBroadway.Splunk.Producer","doc":"GenStage Producer for a Splunk Event Stream.\nBroadway producer acts as a consumer for Splunk report or alerts.","ref":"OffBroadway.Splunk.Producer.html"},{"type":"module","title":"Producer Options - OffBroadway.Splunk.Producer","doc":"* `:name` - Required. The report or alert name for the Splunk job we want to consume events from.\n\n* `:jobs` - Which jobs to add to the initial queue. Possible values are: `:all`, `:new`, `:latest`. The default value is `:all`.\n\n* `:receive_interval` (`t:non_neg_integer/0`) - The duration (in milliseconds) for which the producer waits before\n  making a request for more messages. The default value is `5000`.\n\n* `:refetch_interval` (`t:non_neg_integer/0`) - The duration (in milliseconds) to wait before fetching new jobs to be processed. The default value is `60000`.\n\n* `:only_new` (`t:boolean/0`) - *This option is deprecated. Use jobs: :new instead.* If set to `true`, the pipeline will skip adding any existing jobs to the initial queue.\n\n* `:only_latest` (`t:boolean/0`) - *This option is deprecated. Use jobs: :latest instead.* If set to `true`, the pipeline will only add the most recent job to the initial queue.\n\n* `:shutdown_timeout` (`t:timeout/0`) - The duration (in milliseconds) Broadway should wait before timing out when\n  trying to stop the pipeline. The default value is `:infinity`.\n\n* `:on_success` (`t:atom/0`) - Configures the acking behaviour for successful messages. See the \"Acknowledgements\"\n  section below for all the possible values. The default value is `:ack`.\n\n* `:on_failure` (`t:atom/0`) - Configures the acking behaviour for failed messages. See the \"Acknowledgements\"\n  section below for all the possible values. The default value is `:noop`.\n\n* `:splunk_client` - A module that implements the `OffBroadway.Splunk.Client` behaviour.\n  This module is responsible for fetching and acknowledging the messages\n  from Splunk. All options passed to the producer will also be forwarded to\n  the client. The default value is `OffBroadway.Splunk.SplunkClient`.\n\n* `:config` (`t:keyword/0`) - A set of config options that overrides the default config for the `splunk_client`\n  module. Any option set here can also be configured in `config.exs`. The default value is `[]`.\n\n  * `:base_url` (`t:String.t/0`) - Base URL to Splunk instance.\n\n  * `:api_token` (`t:String.t/0`) - API token used to authenticate on the Splunk instance.\n\n  * `:api_version` - Some API endpoints are [available](https://docs.splunk.com/Documentation/Splunk/9.0.3/RESTREF/RESTsearch)\n    in multiple versions. Sets the API version to use (where applicable). The default value is `\"v2\"`.\n\n  * `:max_events` - If set to a positive integer, automatically shut down the pipeline after consuming\n    `max_events` messages from the Splunk API.","ref":"OffBroadway.Splunk.Producer.html#module-producer-options"},{"type":"module","title":"Acknowledgements - OffBroadway.Splunk.Producer","doc":"You can use the `on_success` and `on_failure` options to control how messages are\nacknowledged. You can set these options when starting the Splunk producer or change\nthem for each message through `Broadway.Message.configure_ack/2`. By default, successful\nmessages are acked (`:ack`) and failed messages are not (`:noop`).\n\nThe possible values for `:on_success` and `:on_failure` are:\n\n  * `:ack` - acknowledge the message. Splunk does not have any concept of acking messages,\n    because we are just consuming messages from a web api endpoint.\n    For now we are just executing a `:telemetry` event for acked messages.\n\n  * `:noop` - do not acknowledge the message. No action are taken.","ref":"OffBroadway.Splunk.Producer.html#module-acknowledgements"},{"type":"module","title":"Telemetry - OffBroadway.Splunk.Producer","doc":"This library exposes the following telemetry events:\n\n  * `[:off_broadway_splunk, :receive_jobs, :start]` - Dispatched before fetching jobs\n    from Splunk.\n\n    * measurement: `%{time: System.monotonic_time}`\n    * metadata: `%{name: string, count: integer}`\n\n  * `[:off_broadway_splunk, :receive_jobs, :stop]` - Dispatched when fetching jobs from Splunk\n    is complete.\n\n    * measurement: `%{time: native_time}`\n    * metadata: `%{name: string, count: integer}`\n\n  * `[:off_broadway_splunk, :receive_jobs, :exception]` - Dispatched after a failure while fetching\n    jobs from Splunk.\n\n    * measurement: `%{duration: native_time}`\n    * metadata:\n\n      ```\n      %{\n        name: string,\n        reason: reason,\n        stacktrace: stacktrace\n      }\n      ```\n\n  * `[:off_broadway_splunk, :process_job, :start]` - Dispatched before starting to process\n    messages for a job.\n\n    * measurement: `%{time: System.system_time}`\n    * metadata: `%{name: string, sid: string}`\n\n  * `[:off_broadway_splunk, :process_job, :stop]` - Dispatched after all messages have been\n    processed for a job.\n\n    * measurement: `%{time: System.system_time, processed_events: integer, processed_requests: integer}`\n    * metadata: `%{name: string, sid: string}`\n\n  * `[:off_broadway_splunk, :receive_messages, :start]` - Dispatched before receiving\n    messages from Splunk.\n\n    * measurement: `%{time: System.monotonic_time}`\n    * metadata: `%{name: string, sid: string, demand: integer}`\n\n  * `[:off_broadway_splunk, :receive_messages, :stop]` - Dispatched after messages have been\n    received from Splunk and \"wrapped\".\n\n    * measurement: `%{time: native_time}`\n    * metadata:\n\n      ```\n      %{\n        name: string,\n        sid: string,\n        received: integer,\n        demand: integer\n      }\n      ```\n\n  * `[:off_broadway_splunk, :receive_messages, :exception]` - Dispatched after a failure while\n    receiving messages from Splunk.\n\n    * measurement: `%{duration: native_time}`\n    * metadata:\n\n      ```\n      %{\n        name: string,\n        sid: string,\n        demand: integer,\n        reason: reason,\n        stacktrace: stacktrace\n      }\n      ```\n\n  * `[:off_broadway_splunk, :receive_messages, :ack]` - Dispatched when acking a message.\n\n    * measurement: `%{time: System.system_time, count: 1}`\n    * meatadata:\n\n      ```\n      %{\n        name: string,\n        receipt: receipt\n      }\n      ```","ref":"OffBroadway.Splunk.Producer.html#module-telemetry"},{"type":"module","title":"OffBroadway.Splunk.SplunkClient","doc":"Default Splunk client used by `OffBroadway.Splunk.Producer` to receive data from\nSplunk.\nThis module implements the `OffBroadway.Splunk.Client` and `Broadway.Acknowledger`\nbehaviours which define callbacks for receiving and acknowledging messages.\nSince Splunk does not have any concept of acknowledging consumed messages, we need\nto keep track of what messages that are consumed ourselves (more on that later).\n\nThe default Splunk client uses the Splunk Web API for receiving messages and is\nimplemented using the [Tesla](https://hexdocs.pm/tesla/readme.html) library. Tesla\nis a HTTP client abstraction library which let's us easily select from a range of\nHTTP adapters. Please see the Tesla [documentation](https://hexdocs.pm/tesla/readme.html#adapters)\nfor more information.","ref":"OffBroadway.Splunk.SplunkClient.html"},{"type":"function","title":"OffBroadway.Splunk.SplunkClient.client/1","doc":"Returns a `Tesla.Client` configured with middleware.\n\n  * `Tesla.Middleware.BaseUrl` middleware configured with `base_url` passed via `opts`.\n  * `Tesla.Middleware.BearerAuth` middleware configured with `api_token` passed via `opts`.\n  * `Tesla.Middleware.Query` middleware configured with `query` passed via `opts`.\n  * `Tesla.Middleware.JSON` middleware configured with `Jason` engine.","ref":"OffBroadway.Splunk.SplunkClient.html#client/1"},{"type":"extras","title":"OffBroadway.Splunk","doc":"# OffBroadway.Splunk\n\n![pipeline status](https://github.com/Intility/off_broadway_splunk/actions/workflows/elixir.yaml/badge.svg?event=push)\n\nA Splunk consumer for [Broadway](https://github.com/dashbitco/broadway).\n\nBroadway producer acts as a consumer for a given Splunk report or (triggered) alert.\nThe `OffBroadway.Splunk.Producer` process will query Splunk for available jobs for the given\nreport and keep them in a queue. Jobs wil then be processed sequentially (from earliest to latest)\nand passed through the Broadway pipeline.\n\nRead the full documentation [here](https://hexdocs.pm/off_broadway_splunk/readme.html).","ref":"readme.html"},{"type":"extras","title":"Installation - OffBroadway.Splunk","doc":"This package is [available in Hex](https://hex.pm/packages/off_broadway_splunk), and can be installed\nby adding `off_broadway_splunk` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:off_broadway_splunk, \"~> 2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - OffBroadway.Splunk","doc":"The `OffBroadway.Splunk.SplunkClient` tries to read the following configuration from `config.exs`.\n\n```elixir\n# config.exs\n\nconfig :off_broadway_splunk, :splunk_client,\n  base_url: System.get_env(\"SPLUNK_BASE_URL\", \"https://splunk.example.com\"),\n  api_token: System.get_env(\"SPLUNK_API_TOKEN\", \"your-api-token-here\")\n```\n\nOptions for the `OffBroadway.Splunk.SplunkClient` can be configured either in `config.exs` or passed as\noptions directly to the `OffBroadway.Splunk.Producer` module. Options are merged, with the passed options\ntaking precedence over those configured in `config.exs`.\n\n```elixir\n# my_broadway.ex\n\ndefmodule MyBroadway do\n  use Broadway\n\n  alias Broadway.Message\n\n  def start_link(_opts) do\n    Broadway.start_link(__MODULE__,\n      name: __MODULE__,\n      producer: [\n        module:\n          {OffBroadway.Splunk.Producer,\n           name: \"My fine report\",\n           config: [api_token: \"override-api-token\"]}\n      ],\n      processors: [\n        default: []\n      ],\n      batchers: [\n        default: [\n          batch_size: 500,\n          batch_timeout: 5000\n        ]\n      ]\n    )\n  end\n\n  ...callbacks...\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Processing messages - OffBroadway.Splunk","doc":"In order to process incoming messages, we need to implement some callback functions.\n\n```elixir\ndefmodule MyBroadway do\n  use Broadway\n\n  alias Broadway.Message\n\n  ...start_link...\n\n  @impl true\n  def handle_message(_, %Message{data: data} ,_) do\n    message\n    |> Message.update_data(fn -> ...whatever... end)\n  end\n\n  @impl true\n  def handle_batch(_batcher, messages, _batch_info, _context) do\n    IO.puts(\"Received a batch of #{length(messages)} messages!\")\n    messages\n  end\nend\n```\n\nFor the sake of the example, we're not really doing anything here. Whenever we're receiving a batch of messages, we just prints out a\nmessage saying \"Received a batch of messages!\", and for each message we run `Message.update_data/2` passing a function that can process\nthat message ie. by doing some calculations on the data or something else.","ref":"readme.html#processing-messages"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"2.1.3 - Patch release - Changelog","doc":"_Released 2024-08-20_\n\n- Replace `:only_new` and `:only_latest` options with a `:jobs` option. Set `:jobs` to `:new` to only process new jobs, and `:latest` \nto only process the latest available job.\n- Add deprecation warnings to `:only_new` and `:only_latest` option. The options are still accepted but will be removed in the\nnext major release.\n- If the client receives an `{:error, reason}` tuple, reschedule another fetch instead of blowing up.","ref":"changelog.html#2-1-3-patch-release"},{"type":"extras","title":"2.1.1 - Patch release - Changelog","doc":"_Released 2023-12-28_\n\nMinor fixes \n\n- Generate unique message IDs using the `_bkt` and `_cd` metadata fields.","ref":"changelog.html#2-1-1-patch-release"},{"type":"extras","title":"2.1.1 - Patch release - Changelog","doc":"_Released 2023-12-07_\n\nMinor fixes\n\n- Make the producer accept `{:error, reason}` tuples as response from API client. Whenever this occurs,\n   simply reschedule a fetch after `receive_interval` milliseconds.","ref":"changelog.html#2-1-1-patch-release"},{"type":"extras","title":"2.1.0 - Minor release - Changelog","doc":"_Released 2023-12-06_\n\nOptions\n\n- Add new option `only_new` to skip consuming any existing jobs. The pipeline will ignore currently known\n  jobs, and only consume new jobs that arrives after the pipeline has started.\n- Add new option `only_latest` to only consume the most recent job for given report or alert.\n\nOther\n\n- Remove `OffBroadway.Splunk.Queue` GenServer process and keep the queue in the producer.\n  This makes the producer process fully self-contained and we no longer need to communicate with\n  another process to know what job we should produce messages for.\n- New telemetry events `[:off_broadway_splunk, :process_job, :start]` and `[:off_broadway_splunk, :process_job, :stop]`\n  are generated whenever a new job is started.\n- Log error and return empty list of messages when receiving an `{:error, reason}` tuple while trying to fetch\n  messages from Splunk.","ref":"changelog.html#2-1-0-minor-release"},{"type":"extras","title":"2.0.0 - Major release - Changelog","doc":"_Released 2023-05-23_\n\nThis almost a complete rewrite and is **incompatible** with the `v1.x` branch.\nInstead of targeting a specific `SID` to produce messages for, this release is focused around producing messages\nfrom Splunk Reports or (triggered) Alerts.\nThis is a more efficient way to prepare data for export by Splunk, and produces more predictable messages both\nin terms of when they are available and the structure of the data in the messages.\n\nInstead of passing a `SID` to the producer option, simply pass the `name` for your report or alert. The\n`OffBroadway.Splunk.Queue` process will query the Splunk Web API and fetch a list of available jobs that can produce\nmessages for the given report or alert.\n\nOptions\n\n- Replace `sid` option with `name`. Pipelines should now be given the name of a report or alert.\n- Remove `endpoint` option. All messages will be downloaded using the `results` endpoint.\n- Remove `offset` option, as it is only available for the `events` endpoint.\n- Add `refetch_interval`option. This is the amount in milliseconds the `OffBroadway.Splunk.Queue` process will\n  wait before refetching the list of available jobs.\n\nOther\n\n- Add `OffBroadway.Splunk.Queue` GenServer process that will start as part of the pipeline supervision tree.\n- Remove `OffBroadway.Splunk.Leader` GenServer process as it is not usable anymore.\n- Refactored `OffBroadway.Splunk.Producer` and `OffBroadway.Splunk.SplunkClient` to new workflow.\n- Updated `telemetry` events to new workflow.","ref":"changelog.html#2-0-0-major-release"},{"type":"extras","title":"1.2.4 - Patch release - Changelog","doc":"_Released 2023-04-20_\n\nBug fixes\n\n- Using `state.is_done` proved unreliable when consuming certain jobs. Replaced calculation of retry timings\n  to be based on `receive_interval`.\n- Fixed typings for `OffBroadway.Splunk.Leader` struct.","ref":"changelog.html#1-2-4-patch-release"},{"type":"extras","title":"1.2.3 - Patch release - Changelog","doc":"_Released 2023-04-05_\n\nMinor fixes\n\n- Remove `Tesla.Middleware.Logger` from default `OffBroadway.Splunk.SplunkClient` tesla client because\n  of too much noise.","ref":"changelog.html#1-2-3-patch-release"},{"type":"extras","title":"1.2.2 - Patch release - Changelog","doc":"_Released 2023-04-03_\n\nMinor fixes\n\n- Filter `authorization` headers for `Tesla.Middleware.Logger`\n- Replace some enumerations with streams","ref":"changelog.html#1-2-2-patch-release"},{"type":"extras","title":"1.2.1 - Patch release - Changelog","doc":"_Released 2023-03-28_\n\nUpgrade accepted dependencies\n\n- Accept `nimble_options` version `v1.0`","ref":"changelog.html#1-2-1-patch-release"},{"type":"extras","title":"1.2.0 - Minor release - Changelog","doc":"_Released 2023-01-23_\n\nAdded new options\n\n- `api_version` - Configures if messages should be produced from the `v1` or `v2` versioned API endpoints.\n\nUpgrade accepted dependencies\n\n- Accept `telemetry` version `1.1` or `1.2`\n- Accept `tesla` version `1.4` or `1.5`","ref":"changelog.html#1-2-0-minor-release"},{"type":"extras","title":"1.1.1 - Minor release - Changelog","doc":"_Released 2023-01-16_\n\nAdded new options\n\n- `shutdown_timeout` - Configurable number of milliseconds Broadway should wait before timing out when trying to stop\n  the pipeline.\n- `endpoint` - Choose to consume messages using the `events` or `results` endpoint of the Splunk Web API.\n- `offset` - Allow passing a custom initial offset to start consuming messages from. Passing a negative value will\n  cause the pipeline to consume messages from the \"end\" of the results.\n- `max_events` - If set to a positive integer, shut down the pipeline after producing this many messages.","ref":"changelog.html#1-1-1-minor-release"},{"type":"extras","title":"1.1.0 - Initial release - Changelog","doc":"_Released 2022-10-28_\n\nThe first release targeted consuming a single SID (Search ID) produced by saving a triggered alert.","ref":"changelog.html#1-1-0-initial-release"},{"type":"extras","title":"LICENSE","doc":"Copyright 2022 Intility\n\n                                 Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]},"content_type":"text/markdown"}